@@ -1,20 +1,13 @@
 // Filename: descriptionwindow.cpp
 
 #include <QDebug>
-
-#include <QVBoxLayout>
-#include <QHBoxLayout>
-#include <QLabel>
-#include <QPainter>
-#include <QKeyEvent>
-
-#include <QBoxLayout>
+#include <QDesktopServices>
 
 #include "htmlview.h"
 #include "descriptionwindow.h"
 
 DescriptionWindow::DescriptionWindow(const Cache &cache, QWidget *parent) : 
-  CustomWindow("DescriptionWindow", parent) {
+  CustomWindow("DescriptionWindow", parent), m_cache(cache) {
 
   setWindowTitle(cache.description());
 
@@ -22,9 +15,15 @@ DescriptionWindow::DescriptionWindow(const Cache &cache, QWidget *parent) :
   connect(this, SIGNAL(zoomInPressed()), htmlView, SLOT(zoomIn()));
   connect(this, SIGNAL(zoomOutPressed()), htmlView, SLOT(zoomOut()));
 
+  addMenuEntry(tr("Open in browser"), this, SLOT(openCacheUrl()));
+
   setCentralWidget(htmlView);
 }
 
 DescriptionWindow::~DescriptionWindow() {
   qDebug() << __FUNCTION__;
 }
+
+void DescriptionWindow::openCacheUrl() {
+  QDesktopServices::openUrl(m_cache.url());
+}
