@@ -7,7 +7,7 @@
 #include <math.h>
 
 #include "mapwidget.h"
-#include "size.h"
+#include "config.h"
 
 #if(QTM_VERSION < 0x010100)
 #error "QtMobility version 1.1 or higher required!"
@@ -37,7 +37,7 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
   m_gpsLocked = settings.value("GpsLocked", false).toBool();
   m_precisionAlpha = settings.value("PrecisionAlpha", 32).toInt();
   settings.endGroup();
-  m_iconLoader = new IconLoader(Size::MAP_WIDGET_ICON_SIZE);
+  m_iconLoader = new IconLoader(Config::MAP_WIDGET_ICON_SIZE);
   
   // create the control overlays
   this->m_mapOverlay = new MapOverlay(this);
@@ -61,7 +61,7 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
 		  QPoint(-m_iconLoader->size()/2, -m_iconLoader->size()/2),
 		  *markerIcon);
 
-    m_gpsMarker->setZValue(2);
+    m_gpsMarker->setZValue(3);
     m_gpsMarker->setVisible(false);
     addMapObject(m_gpsMarker);
   }
@@ -69,7 +69,7 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
   m_gpsPrecision = new QGeoMapCircleObject();
   m_gpsPrecision->setPen(QPen((QColor(0, 0, 0, m_precisionAlpha))));
   m_gpsPrecision->setBrush(QBrush(QColor(0, 0, 0, m_precisionAlpha/2)));
-  m_gpsPrecision->setZValue(1);
+  m_gpsPrecision->setZValue(2);
   addMapObject(m_gpsPrecision);
 
   // install timer to delay cache reload requests
@@ -228,13 +228,19 @@ void MapWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
   // check if we clicked a cache
   bool clickedBubble = false;
   QString clickedCache;
+  Waypoint clickedWaypoint;
 
   QList<QGeoMapObject*> objects = mapObjectsAtScreenPosition(m_downPos);
   if (objects.length() > 0) {
     for (int i = objects.length()-1; i >= 0; i--) {
       if ((objects[i]->objectName() == "cache") && clickedCache.isEmpty())
 	clickedCache = objects[i]->property("name").toString();
-      
+      if ((objects[i]->objectName() == "waypoint") && clickedCache.isEmpty()) {
+	clickedCache = objects[i]->property("cache").toString();
+	clickedWaypoint = 
+	  static_cast<MapWaypointObject *>(objects[i])->waypoint();
+      }
+
       if (objects[i]->objectName() == "bubble") 
 	clickedBubble = true;
     }
@@ -243,8 +249,20 @@ void MapWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
   if(!clickedBubble) {
     hideBubble();
 
-    if(!clickedCache.isEmpty()) 
-      emit cacheClicked(clickedCache);
+    if(!clickedCache.isEmpty()) {
+      m_selectedCache = clickedCache;
+      
+      /* update caches to show waypoints of selected cache if present */
+      emit mapChanged();
+ 
+      if(!clickedWaypoint.coordinate().isValid()) 
+	emit cacheClicked(clickedCache);
+      else {
+	qDebug() << __FUNCTION__ << "User clicked waypoint" <<
+	  clickedWaypoint.name() << "of cache" << clickedCache;
+
+      }
+    }
   }
 }
 
@@ -270,15 +288,20 @@ void MapWidget::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
   pan(d.x(), d.y());
 }
 
-MapWidget::MapCacheObject::MapCacheObject(IconLoader *loader, const Cache &cache) :
+MapWidget::MapCacheObject::MapCacheObject(IconLoader *loader, const Cache &cache, bool highlight) :
   QGeoMapPixmapObject(cache.coordinate(), 
 		      QPoint(-loader->size()/2, -loader->size()/2),
 		      *loader->load(cache)) {
-  
+
+  if(highlight) {
+    QPixmap *hlPix = loader->loadHighlight(cache);
+    setOffset(QPoint(-hlPix->width()/2, -hlPix->height()/2));
+    setPixmap(*hlPix);
+  }
+
   setZValue(0);  // below gps marker and bubble
   setObjectName("cache");
   setProperty("name", cache.name());
-  setProperty("description", cache.description());
 };
 
 // the pixmap object seems to actually check for clicks onto the bitmap. thus
@@ -288,6 +311,22 @@ bool MapWidget::MapCacheObject::contains(const QGeoCoordinate &coo) const {
   return boundingBox().contains(coo);
 }
 
+MapWidget::MapWaypointObject::MapWaypointObject(IconLoader *loader, 
+		const QString &cacheName, const Waypoint &waypoint) :
+  QGeoMapPixmapObject(waypoint.coordinate(),
+		      QPoint(-loader->size()/3, -loader->size()/3),
+		      *loader->load(waypoint.iconFile(), 2*loader->size()/3)) {
+  
+  setZValue(1);  // below gps marker and bubble, but above caches
+  setObjectName("waypoint");
+  setProperty("cache", cacheName);
+  m_wpt = waypoint;
+};
+
+Waypoint MapWidget::MapWaypointObject::waypoint() const {
+  return m_wpt;
+}
+
 void MapWidget::updateCaches(const CacheList &cacheList) {
   static bool tooOld = false;
 
@@ -311,8 +350,22 @@ void MapWidget::updateCaches(const CacheList &cacheList) {
 
   // draw all new caches
   QList<Cache>::const_iterator i;
-  for( i = cacheList.begin(); i != cacheList.end(); ++i ) 
-    m_cacheGroup->addChildObject(new MapCacheObject(m_iconLoader, *i));
+  for( i = cacheList.begin(); i != cacheList.end(); ++i ) {
+    bool isSelected = (i->name() == m_selectedCache);
+
+    // highlight if cache has waypoints
+    m_cacheGroup->addChildObject(new MapCacheObject(m_iconLoader, *i, 
+		    isSelected && i->waypoints().size() > 0));
+				 
+    // also draw waypoints of selected cache
+    if(isSelected) 
+      m_selectedWaypoints = i->waypoints();
+  }
+
+  // draw all waypoints from list
+  foreach(Waypoint wpt, m_selectedWaypoints)
+    m_cacheGroup->addChildObject(new MapWaypointObject(m_iconLoader, m_selectedCache, wpt));
+
 
   // restore previous bubble
   if(bubbleCache.coordinate().isValid()) {
