@@ -65,10 +65,18 @@ class MapWidget : public QGraphicsGeoMap {
  private:
   class MapCacheObject : public QGeoMapPixmapObject {
   public:
-    MapCacheObject(IconLoader *, const Cache &);
+    MapCacheObject(IconLoader *, const Cache &, bool);
     bool contains( const QGeoCoordinate &) const;
   };
 
+  class MapWaypointObject : public QGeoMapPixmapObject {
+  public:
+    MapWaypointObject(IconLoader *, const QString &, const Waypoint &);
+    Waypoint waypoint() const;
+  private:
+    Waypoint m_wpt;
+  };
+
   // this should actually be much lower on non-touch devices
   static const int DRAG_FUZZ = 30;
   
@@ -84,6 +92,8 @@ class MapWidget : public QGraphicsGeoMap {
   bool m_posIsValid;
   bool m_gpsLocked;
   int m_precisionAlpha;
+  QString m_selectedCache;
+  QList<Waypoint> m_selectedWaypoints;
 
   // make sure map is only updated if it's dragged
   // significantly
